# This file is read by either "tclmake" or "tkmake" and is used to
# build all demo programs associated with Embedded Tk.
#

set Instructions \
{                                  INSTRUCTIONS

Follow these steps to build the demonstration programs for Embedded Tk:

  *  Select the Options/Edit-Configuration menu item to bring up the 
     configuration editor.

  *  Make configuration changes appropriate to your needs and the requirements 
     of your system.

  *  Press the "Save" or "OK" button on the configuration editor to save the 
     configuration.

  *  Select a program to build under the Make menu.  Choose "all" to build 
     everything.  Choose "clean" to delete everything that has been built 
     previously.  Note that some demonstration programs will not compile on 
     some architectures.

  *  You can reload these instructions by selecting the Help/Instructions 
     menu option.

Please e-mail comments and/or bug-reports to drh@vnet.net.
}

set Parameter(ETFLAGS) {\
Any command line options used by ``et2c''.  Set to one or more\
of the following: -dynamic, -nocompress, -K+R}
set ETFLAGS {}

set Parameter(TKLIB) {\
Flags given to the C compile that cause it to link with the Tcl/Tk library}
set TKLIB {-ltk -ltcl -lX11 -lm}

set Parameter(STRIP) {\
Set to 1 to cause the command ``strip'' is run over the executable after\
it is compiled}
set STRIP 1

set Parameter(CC) {Name of the C compiler}
set CC gcc

set Parameter(CFLAGS) {Flags given to the C compiler}
set CFLAGS {-O}

set Parameter(ETSRC) {\
The name of the source code file for the ``et.o'' library.  Use et36.c\
for Tk version 3.6.  Use et40.c for Tk version 4.0. \
Use et41.c for Tk version 4.1.  And so forth.}
set ETSRC et40.c

set Parameter(ETSCRIPTDIRS) {\
Use -I switches to specify the directories that contain the Tcl/Tk\
startup scripts. This is needed so that the startup scripts can be\
compiled into the et.o library.}
set ETSCRIPTDIRS {-I/usr/lib/tcl -I/usr/lib/tk}

set Parameter(ETINCLUDEDIRS) {\
Use -I switches to specify the directories that contain the Tcl/Tk\
header files ``tcl.h'' and ``tk.h''.}
set ETINCLUDEDIRS {-I/usr/include/tcl -I/usr/include/tk}

set allapps {\
bell birdseye browser color dclock etwish fontchooser hello perfmon\
runscript tkdiff tkedit tkterm\
}

Make all from $allapps

Make et2c from et2c.c using {
  Shell {$CC $CFLAGS -o $target et2c.c}
}

Make et.o from {et2c et36.c et40.c et41.c} using {
  Shell {./et2c $ETFLAGS $ETSCRIPTDIRS $ETSRC >et.c}
  Shell {$CC $CFLAGS $ETINCLUDEDIRS -o $target -c et.c}
}

foreach t {bell birdseye dclock etwish hello runscript www httpd} {
  Make $t from "et2c et.o $t.c" using [format {
    Shell {./et2c $ETFLAGS %s.c >temp.c}
    Shell {$CC $CFLAGS $ETINCLUDEDIRS -o $target temp.c et.o $TKLIB}
    Shell {rm -f temp.c}
    if {$STRIP} {Shell {strip $target}}
  } $t]
}

foreach t {color} {
  Make ${t}help.tcl from "$t.help help2tk" using [format {
    Shell {help2tk %s.help >$target}
  } $t]
  Make $t from "et2c et.o ${t}help.tcl $t.tcl help.tcl" using [format {
    Shell {./et2c $ETFLAGS %s.c >temp.c}
    Shell {$CC $CFLAGS $ETINCLUDEDIRS -o $target temp.c et.o $TKLIB}
    Shell {rm -f temp.c}
    if {$STRIP} {Shell {strip $target}}
  } $t]
}

Make help2tk from help2tk.c using {
  Shell {$CC $CFLAGS -o $target help2tk.c}
}

foreach t {browser fontchooser perfmon tkdiff tkedit linuxstat} {
  Make $t from "et2c et.o $t.c $t.tcl" using [format {
    Shell {./et2c $ETFLAGS %s.c >temp.c}
    Shell {$CC $CFLAGS $ETINCLUDEDIRS -o $target temp.c et.o $TKLIB}
    Shell {rm -f temp.c}
    if {$STRIP} {Shell {strip $target}}
  } $t]
}

Make tkterm from {et.o tkterm.o getpty.o} using {
  Shell {$CC $CFLAGS -o $target tkterm.o getpty.o et.o $TKLIB}
  if {$STRIP} {
    Shell {strip $target}
  }
}
Make tkterm.o from {tkterm.c et2c tkterm.tcl} using {
  Shell {./et2c $ETFLAGS tkterm.c >temp.c}
  Shell {$CC $CFLAGS $ETINCLUDEDIRS -o $target -c temp.c}
  Shell {rm -f temp.c}
}
Make getpty.o from getpty.c using {
  Shell {$CC $CFLAGS -o $target -c getpty.c}
}

Make dummydl.o from dummydl.c using {
  Shell {$CC $CFLAGS -o $target -c $source}
}

Make clean using {
  Shell {rm -f *.o temp.c $allapps et2c et.o}
}
