#! /usr/bin/wish
#
# This TCL script (for Tk version 4.1 or later) implements a very very
# simple web browser.  No HTML translation is done -- the retrieved documents
# are displayed as raw text.
#
wm title . {Demo Web Browser}
wm iconname . {DemoWebBrowser}
option add *highlightThickness 0

#
# Construct the main menu
#
frame .mbar -bd 2 -relief raised
pack .mbar -side top -fill x -expand 1
menubutton .mbar.file -text File -underline 0 -menu .mbar.file.menu
menubutton .mbar.edit -text Edit -underline 0 -menu .mbar.edit.menu
menubutton .mbar.help -text Help -underline 0 -menu .mbar.help.menu
pack .mbar.file .mbar.edit .mbar.help -side left -pady 0 -padx 10
set m [menu .mbar.file.menu]
$m add command -label Exit -underline 1 -command exit
set m [menu .mbar.edit.menu]
$m add command -label Load -underline 0 -command LoadDocument
set m [menu .mbar.help.menu]
$m add command -label Instructions -underline 0
$m add command -label {About This Program} -underline 0

#
# Construct the entry for the URL
#
frame .url -bd 1 -relief raised
pack .url -side top -fill x -expand 1
frame .url.f 
pack .url.f -padx 5 -pady 5 -expand 1 -anchor w
label .url.f.http -text http:
entry .url.f.e -width 65 -font fixed -bd 1 -relief sunken
bind .url.f.e <Return> {LoadDocument; break}
pack .url.f.http -side left 
pack .url.f.e -side left -expand 1 -fill x

#
# Construct the display area
#
frame .x -bd 1 -relief raised
pack .x -side top -fill both -expand 1
text .x.t -width 80 -height 24 -font fixed -bd 0 -yscrollcommand {.x.sb set}
scrollbar .x.sb -command {.x.t yview}
pack .x.t -side left -fill both -expand 1
pack .x.sb -side left -fill y

set sock {}

#
# Set the input focus on the URL entry
#
update
focus .url.f.e

#
# Open a connect to the web document described in the HTTP entry
#
proc LoadDocument {} {
  set url [.url.f.e get]
  .x.t delete 1.0 end
  if {![regexp {^(http:)?//([^/:]*)(:[^/]*)?(.*)$} $url x x ipaddr port file]} {
    .x.t insert end "Bad URL.  Please use this format:\n\n"
    .x.t insert end "   //domain.name:port/dir/file.html\n"
    return
  }
  set port [string trimleft $port :]
  if {"$port"==""} {set port 80}
  global sock
  if {"$sock"!=""} {catch {close $sock}}
  if {[catch {set sock [socket $ipaddr $port]} errmsg]} {
    .x.t insert end "404 ERROR\n\n  "
    .x.t insert end $errmsg
    set sock {}
    return
  }
  puts $sock "GET $file HTTP/1.0\n"
  flush $sock
  fconfigure $sock -blocking 0
  fileevent $sock readable "GetText $sock"
}

#
# Actually load text into the text widget from the socket
#
proc GetText {s} {
  if {"$s"=="" || [eof $s]} {
    catch {close $s}
    set s {}
    return
  }
  while {"[set x [gets $s]]"!=""} {
    .x.t insert end $x\n
  }
  if {[eof $s]} {
    close $s
    set s {}
  }
}
