#ifndef consistency_h
#define consistency_h

#ifndef comms_h
#include "comms.h"
#endif

#define MSYNC       2000
#define BSYNC       2001
#define ENTRY       2002
#define ALL_CLIENTS 10000

int con_protocol;

typedef struct sched *sched_p;

typedef struct sched {
  int client_id;
  int time;
  sched_p next;
} sched_t;

sched_t *el_front;		/* Exchange-list front. 
				 This is a pointer to a
				 list of processes that
				 must be synchronized 
				 with.*/

/* Keep list of clients that have sent us SYNC messages and are
   awaiting our reply with a SYNC message.*/
sync_t sync_list[NUM_TEAMS];

/* Maintain store of flags denoting which of this team's robots
   have generated events that must be sent to each of the other
   teams (clients). This data structure is req'd by the lookahead
   protocol. */
int event_buffer[NUM_TEAMS][MAX_ROBOTS];

dlock_t mgr_lock[FIELDX][FIELDY];
dlock_t client_lock[FIELDX][FIELDY];

int lock_granted;

writers_t *addToLockList (writers_t *front, 
			  writers_t *w);
void delFromLockList (writers_t **front);
void acquireLockHandler (int cid, int flag, dlock_t *l);
void releaseLockHandler (int cid, dlock_t *l);
void sendAcqLock (int cid, dlock_t *l);
void sendRelLock (int cid, dlock_t *l);
int acquireLock (int cid,
		 dlock_t *l, 
		 int x,
		 int y);

/* 'lock_tt' is a lock translation table. It translates a 'lock_id'
   into an object coordinate in the shared 'world'. This should
   really be a hash table, since each lock-manager is responsible
   for only a subset of lockable objects. Only in a centralized 
   system would we require a tranlation table to be large enough
   for every possible lockable object. 'lock_tt' may be hashed at
   some later stage but for now it is a fast (O(1)) array. */
coordinate lock_tt[FIELDX*FIELDY];

void setupConProto (int protocol);

int *calcShortestDist (int client_number,
		       int *min_dist);
void sortExchangeTimes (int *min_dist,
			sched_t *exchange,
			int num);
void addExchangeTimeToList (int client_id,
		      int exchange_time,
		      sched_t **front);
void delExchangeTimeFromList (int client_id,
			      sched_t **front);

#endif






