#ifndef measure_h
#define measure_h

/*
 * MEASURE.H: Intended for measuring resource usage, like 
 * time or number of messages for any part of users' code.
 */

#include "general.h"

#define NUM_EXPERIMENTS 100

/* Here are the names of some experiments: */
#define SEND_EVENT   10
#define ACQUIRE_LOCK 20
#define RELEASE_LOCK 30
#define GET_DATA     40
#define S_FUNCTION   50

/* for fopen, printf */
#include <stdio.h>

/* for getrusage() system call */
#include <sys/time.h>
#include <sys/resource.h>

/* for string operations */
#include <string.h>

extern const char *EXPM_BaseFilename;  /* base file name */
extern u_long sendMsgCount;
extern u_long walkCount;	/* Count how many times a walk 
				   event occurs. */
typedef struct _file_info{
  char name[80];  /* File name extension.*/
  FILE *fp;       /* File pointer.       */
} file_info;

/* files for keeping results for all experiments */
extern file_info EXPM_Files [NUM_TEAMS][NUM_EXPERIMENTS]; 

/* structure that keeps acumulated and avg. results through measuremets */
typedef struct _EXPM_rusage{ 
  int cycle;                     /* number of measurements */

  struct rusage start_rusage;    /* start rusage, in one measurement */
  struct rusage stop_rusage;     /* stop rusage, in one measurement*/
  struct timeval start_time;     /* start time, in one measurement*/
  struct timeval stop_time;      /* stop time, in one measurement*/

  double ac_ru_utime;            /* acumulated user time used */
  double ac_ru_stime;            /* acumulated system time used */
  double av_ru_utime;            /* average user time used */
  double av_ru_stime;            /* average system time used */

  int    ac_ru_msgsnd;           /* acumulated messages sent */
  int    ac_ru_msgrcv;           /* acumulated messages received */
  int    av_ru_msgsnd;           /* average messages sent */
  int    av_ru_msgrcv;           /* average messages received */
  double total_time;             /* average total time used */
  u_long startMsgCount;
  u_long stopMsgCount;
  u_long sendMsgCount;
  u_long startWalkCount;
  u_long stopWalkCount;
  u_long walkCount;
} EXPM_rusage;


/* one structure per exp. */
extern EXPM_rusage EXPM_rusages [NUM_TEAMS][NUM_EXPERIMENTS]; 


/* function prototypes */
void EXPM_Initialize (int con_protocol, 
		      int range, 
		      int num_clients,
		      int id, 
		      int exp_no); 
void EXPM_Start (int id, int exp_no);
void EXPM_Stop (int id, int exp_no); 
void EXPM_Close (int id, int exp_no);
#endif





