#include "scheduler.h"
#include "consistency.h"

#include <sys/time.h>
#include <signal.h>

extern FILE *debug_file;

extern int sched_num;
extern int sound;
extern int team_size;
extern char *opmode; 
extern char *pause_string;
extern char * sector;
extern int num_clients;

extern void update_scores (int team_number, int robot_number);
extern void redrawGoal (char *sector, coordinate goal);
extern void sendDataReq (datareq_t (*client_list)[MAX_RANGE], 
			 int range);
int paused;
int stop;
extern int free_data_token;

/* The clients have their own function, independent of server's, 
   for manipulating client-only data structures. */
void getRobotInfo (int x, int y, int *team_number, int *robot_number) {
  
  int i, j;

  for (i = 0; i < NUM_TEAMS; i++) 
    for (j = 0; j < MAX_ROBOTS; j++) {
      if ((robot_position[i][j].x == x) &&
	  (robot_position[i][j].y == y))
	if (robot_life[i][j] > 0) {
	*team_number = i;
	*robot_number = j;
	return;  /* Force a double break. */
      }
    }
}


/* A very dull signal-catcher. */
void sig_alrm () {
  return;
}


void snooze (unsigned long interval) {
  struct itimerval value;

  value.it_value.tv_usec = interval;
  value.it_value.tv_sec = 0;
  value.it_interval.tv_usec = 0;
  value.it_interval.tv_sec = 0;

  if (signal (SIGALRM, sig_alrm) == SIG_ERR)
    fprintf (stderr, "Signal error.\n");
  if (setitimer (ITIMER_REAL, &value, NULL) != 0) {
    perror ("Battle(snooze): Error in timer setting");
    shutdown (1);
  }
  
  pause (); /* Wait for caught signal. */

  return;
}


void update_time () {
  current_time++;
  ET( set current_time %d(current_time));
}


void missileControl () {
  int i;

  for (i = 0; i < MAX_MISSILES; i++)
    if (missile[i].status == MISSILE_ACTIVE)
      update_missile (i);
}


void robotControl (char *opmode) {
  int i = 0, j, k, l, x, y, team_number;
  int all_finished = FALSE;
  int finished [MAX_ROBOTS];
  datareq_t client_list[NUM_TEAMS][MAX_RANGE];
  send_t how;
  task *next_job;

  /* Keep note of who we need to send data to each time
     we generate an event. 
     NOTE: Missile events are always broadcast. */
  int exchange_list[NUM_TEAMS];	

  for (k = 0; k < MAX_ROBOTS; k++)
    finished [k] = FALSE;

  for (k = 0; k < MAX_MISSILES; k++)
    missile[i].status  = MISSILE_INACTIVE;

  do {
    missileControl();

    if (free_data_token) {	
      /* Acquire token to access shared memory.
	 The robotControl() loop can affect the state of
	 shared memory, since one robot-event gets processed
	 every iteration of this loop. */
    
      /* ?? for (j = 0; j < team_size; i = (i+1) % team_size, j++) {*/
      for (y = 0; y < FIELDY; y++)
	for (x = 0; x < FIELDX; x++)
	  if (world[x][y] == FRIENDLY_ROBOT) {
	    getRobotInfo (x, y, &team_number, &i);

	if ((robot_life [id][i] > 0) && (!finished [i])) {
	  /* Look to see if there are any unknown objects around
	     this robot. If there are any, make sure that the actual
	     data about these unknown objects is retrieved from the
	     correct client(s). 
	     for (k = 0; k < FULL_ROTATION; k++) {
	     look (i, sighted_flag, k, MAX_RANGE, client_list);
	     sendDataReq (client_list, MAX_RANGE);
	     } */
	  
	  if (strcmp (opmode, "Computer") == 0) {
	    /* This 'for' loop forces each robot to look N, W, E and S
	       so that blocks are accessed lowest id first. If we use
	       a locking scheme, we always lock blocks lowest id first
	       so can never deadlock! */
#if 0
	    look (i, sighted_flag, 0, MAX_RANGE, NULL); /* ?? Test. */
#endif
	    for (k = 0, l = 1; (k != -1); 
		 k = ((k+FULL_ROTATION-l)%FULL_ROTATION), l++) {
	      /* Look to see what's around us before creating any
		 tasks.*/
	      look (i, sighted_flag, k, MAX_RANGE, NULL);
	      if (k == 2)
		k = -1;	/* Terminate. */
	    }
	    create_task (sched_num, i, sighted_flag, environment);
	  }
	  else /* Potentially process user-input. */
	    while (eventCheck (TRUE, &pause_string));
	  add_to_schedule (sched_num, &buffer [sched_num].front, 
			   buffer [sched_num].back,
			   &buffer [sched_num].status);
	}
	else
	  finished [i] = TRUE;
      }
      i = (i+1) % team_size;
      
      all_finished = TRUE;
      for (k = 0; k < team_size; k++)
	all_finished = all_finished && finished [k];

      if (all_finished) {
	/* Ensure outstanding exchanges are completed with
	   clients that will require data from this client. */
	if (con_protocol != ENTRY) {
#ifdef DEBUG
	  fprintf (debug_file, 
		   "Sending SYNC msg - no free data to dispatch\n");
#endif
	  sendEvent (NULL, NULL, 
		     exchange_list, 
		     all_finished,
		     how = broadcast);
	}
	break;
      }

      if (dispatch (sched_num, &next_job) != CANNOT_DISPATCH) {
	if (con_protocol != ENTRY) {
#ifdef DEBUG
	  fprintf (debug_file, "Sending SYNC msg - dispatched data\n");
#endif
	  sendEvent (next_job->func, next_job->arg, 
		     exchange_list, 
		     all_finished,
		     how = multicast);
	}
	else
	  ECendEvent (next_job->func, next_job->arg);

	if (equal (robot_position [id][next_job->arg->robot_num], goal)) {
	  finished [next_job->arg->robot_num] = TRUE;
	  glorySequence (sector, goal.x, goal.y, FRIENDLY_ROBOT);
	  
#ifdef SOUND
#ifdef SOUNDCARD
	  if (sound)
	    playSoundFile ("goal");
#endif
#endif
	  redrawGoal (sector, goal);
	  world [goal.x][goal.y] = UNCHARTED;
	  update_scores (id, next_job->arg->robot_num);
	}
	free (next_job->arg);
	free (next_job);

	all_finished = TRUE;
	for (k = 0; k < team_size; k++)
	  all_finished = all_finished && finished [k];
	
	if (all_finished) {
	  /* Ensure outstanding exchanges are completed with
	     clients that will require data from this client. */
	  if (con_protocol != ENTRY) {
#ifdef DEBUG
	    fprintf (debug_file, "Sending SYNC msg - all finished\n");
#endif
	    sendEvent (NULL, NULL, 
		       exchange_list, 
		       all_finished,
		       how = broadcast);
	  }
	  break;
	}
      }
      else 
	if (con_protocol != ENTRY) {
#ifdef DEBUG
	  fprintf (debug_file, 
		   "Sending SYNC msg - no free data to dispatch\n");
#endif
	  sendEvent (NULL, NULL, 
		     exchange_list, 
		     all_finished,
		     how = multicast);
	}
    }
    
    if (!free_data_token)
      if (con_protocol != ENTRY) {
#ifdef DEBUG
	fprintf (debug_file, 
		 "Sending SYNC msg - no free data token\n");
#endif
	sendEvent (NULL, NULL, 
		   exchange_list, 
		   all_finished,
		   how = multicast);
      }
    while (eventCheck (TRUE, &pause_string));
  }
  while (stop == FALSE);

#ifdef DEBUG
  fprintf (debug_file,
	   "(Control) Finished do...while loop\n");
#endif

  if (con_protocol != ENTRY)
    sendEvent (NULL, NULL, 
	       exchange_list, 
	       TRUE,
	       how = broadcast);
  
#ifdef DEBUG
  fclose (debug_file);
#endif
}

